/**
 * Copyright (C) 2003-2004 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.server;

import java.io.Serializable;
import java.util.logging.Logger;
import java.util.logging.Level;

import sync4j.framework.logging.Sync4jLogger;

import sync4j.framework.core.SyncML;
import sync4j.framework.core.Sync4jException;

import org.apache.commons.lang.StringUtils;

/** 
 * This class is used for transformer the message in the canonical form.
 *
 */
public class SyncMLCanonizer implements Serializable {

    // ------------------------------------------------------------ Private data
    private static final Logger log = Sync4jLogger.getLogger("engine");
    
    // ------------------------------------------------------------ Constructors
    public SyncMLCanonizer() {
    }

    // ---------------------------------------------------------- Public methods
    
    /**
     * The message input must be canonized every time before call the XML-Java 
     * mapping tool (JiBX) that it parsers and generates the SyncML object.
     *
     * @param message the input XML message
     *
     * @return message the input XML message canonized
     **/
    public String canonize(String message) {
        if (log.isLoggable(Level.FINE)) {
            log.fine("Starting process of canonization");
        }
        
        return metInfNamespaceHandler(message);
    }
   
    /**
     * Replace into message the '&' with "&amp;"
     *
     * @param message the original message xml
     *
     * @return message the message updated
     */
    private String replaceEntity(String message) {
        return StringUtils.replace(message, "&", "&amp;");
    }
    
    /**
     * This is a temporary solution in order to obviate to a JiBX bug: it does 
     * not allow to declare the namespace to level of structure.
     * 
     * @param msg the server response
     *
     * @return the response with namespace correctly added into MetInf element
     */
    private String metInfNamespaceHandler(String msg) {
        int s = 0;
        int e = 0;
        
        StringBuffer response = new StringBuffer();
        while (( e = msg.indexOf("<Meta", s)) >= 0) {
            
            response = response.append(msg.substring(s, e));
            
            int a = msg.indexOf("</Meta>", e);
            String meta = msg.substring(e, a);
            
            meta = meta.replaceAll("<Type>"   , "<Type xmlns='syncml:metinf'>");
            meta = meta.replaceAll("<Format>" , "<Format xmlns='syncml:metinf'>");            
            meta = meta.replaceAll("<Mark>"   , "<Mark xmlns='syncml:metinf'>");
            meta = meta.replaceAll("<Size>"   , "<Size xmlns='syncml:metinf'>");
            meta = meta.replaceAll("<Anchor>" , "<Anchor xmlns='syncml:metinf'>");
            meta = meta.replaceAll("<Version>", "<Version xmlns='syncml:metinf'>");
            meta = meta.replaceAll("<NextNonce>" , "<NextNonce xmlns='syncml:metinf'>");
            meta = meta.replaceAll("<MaxMsgSize>", "<MaxMsgSize xmlns='syncml:metinf'>");
            meta = meta.replaceAll("<MaxObjSize>", "<MaxObjSize xmlns='syncml:metinf'>");
            meta = meta.replaceAll("<EMI>"    , "<EMI xmlns='syncml:metinf'>");
            meta = meta.replaceAll("<Mem>"    , "<Mem xmlns='syncml:metinf'>");
            
            s = a + 7;
            response.append(meta).append("</Meta>");
        }
        return response.append(msg.substring(s, msg.length())).toString();        
    }
}